/* ==========================================================================*/
/* - - - - - - - - - - - - -   M A I N . C   - - - - - - - - - - - - - - - - */
/* ==========================================================================*/
/*	Created   06-12-2004   M. Wheatley                                       */
/*	Modified  09-04-2004   MW                                                */
/*.........................................................................  */
/* Program to Demo Some DSP functionality on DSPic Development board.        */
/*.........................................................................  */
#define __dsPIC30F6014__
/*---------------------------------------------------------------------------*/
/*------------------------> I N C L U D E S <--------------------------------*/
/*---------------------------------------------------------------------------*/
#include <p30f6014.h>
#include <dsp.h>
#include "common.h"
#include "lcd.h"
#include "menu.h"
#include "codec.h"
#include "atod.h"
#include "generators.h"
#include "timers.h"
#include "procdsp.h"

/*---------------------------------------------------------------------------*/
/*----------------> P R O C E S S O R    S P E C I F I C <-------------------*/
/*---------------------------------------------------------------------------*/

/*---------------------------------------------------------------------------*/
/*--------> T Y P E D E F S   U N I O N S    S T R U C T U R E S <-----------*/
/*---------------------------------------------------------------------------*/

/*---------------------------------------------------------------------------*/
/*--------------------> L O C A L   D E F I N E S <--------------------------*/
/*---------------------------------------------------------------------------*/

/*---------------------------------------------------------------------------*/
/*----------------> E X T E R N A L  R E F E R E N C E S <-------------------*/
/*---------------------------------------------------------------------------*/

/*---------------------------------------------------------------------------*/
/*--------------> G L O B A L  M E M O R Y  A L L O C A T I O N <------------*/
/*---------------------------------------------------------------------------*/

/*---------------------------------------------------------------------------*/
/*--------------> L O C A L  M E M O R Y  A L L O C A T I O N <--------------*/
/*---------------------------------------------------------------------------*/
static fractional* pInData;
static fractional* pOutData;

/*---------------------------------------------------------------------------*/
/*--------------------------> P R O T O T Y P E S <--------------------------*/
/*---------------------------------------------------------------------------*/

/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
/*                             ---------                                     */
/*                            | M a i n |                                    */
/*                             ---------                                     */
/*    This is the Main program entry point from boot.                        */
/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
int main(void)
{
	TRISDbits.TRISD0 = 0;	//LED output bits
	TRISDbits.TRISD1 = 0;
	TRISDbits.TRISD2 = 0;
	TRISDbits.TRISD3 = 0;
	PORTD = 0xFFFF;			//All LEDs off
	InitTimers();
	TimeDelay(2);	//let system reset
	InitLcd();
	InitMenu();
	InitAD();
	InitCodec();
	InitGen();
	InitFFT();
	while(1)
	{
		ServiceMenu();
		ServiceLCD();
 		if(Sem.CodecRxDataRdy && Sem.CodecTxDataRdy)
		{	//if both Rx and Tx buffers are ready then process them
			LED1_ON		//here at 35.5mS rate(72000/256)Hz
			Sem.CodecRxDataRdy = FALSE;	
			Sem.CodecTxDataRdy = FALSE;
			pInData = pActiveRxBuf;		//pointer to full half of receive Queue
			pOutData = pActiveTxBuf;	//pointer to empty half of transmit Queue
			switch(InputSrc)	//Select the signal Input source
			{	
				case SRC_CODEC:	//just scale the receive data from the CODEC
					VectorScale(PROC_BLOCK_SIZE, pInData, pInData, CurrentInScale);
					break;	//110uS
				case SRC_SINE:	//create sine wave
					MakeSine(PROC_BLOCK_SIZE, pInData, CurrentInScale);
					break;	//356uS
				case SRC_NOISE:	//create noise
					MakeUniform( PROC_BLOCK_SIZE, pInData, CurrentInScale);
					break;	//2.88mS
				case SRC_SINENOISE:	//make sine and add some noise
					MakeUniform( PROC_BLOCK_SIZE, pOutData, 0x1000);
					MakeSine(PROC_BLOCK_SIZE, pInData, CurrentInScale);
					VectorAdd(PROC_BLOCK_SIZE, pInData, pOutData, pInData);
					break;	//3.3mS
			}
			LED1_OFF
			LED2_ON
			switch(DspProc)	//here to process the pInputData buffer
			{
				case DSP_NONE:	//just echo it to the output buffer
					VectorCopy(PROC_BLOCK_SIZE, pOutData, pInData);
					break;
				case DSP_LPFIR:	// 3.96mS
				case DSP_HPFIR:	// 3.96mS
				case DSP_BPFIR:	// 3.92mS
				case DSP_BSFIR:	// 3.88mS
					ProcFIR( pInData, pOutData);
					break;
				case DSP_SHIFT:	// 10.8mS
					ProcShift( pInData, pOutData);
					break;
				case DSP_LMS:	// 4.8mS
					ProcLMS( pInData, pOutData);
					break;
			}
			LED2_OFF
			LED3_ON
			if(CurrentMenu==MENU_TIME)
			{
				PlotTime(pOutData);
			}
			else if(CurrentMenu==MENU_FREQ)
			{	//make copy since FFT modifies the input buffer
				VectorCopy(PROC_BLOCK_SIZE, pInData, pOutData);
				PlotFreq(pInData);	// 4.16mS for FFT calculation & plot
			}
			LED3_OFF
		}
	}
}
